@@ -16,5 +16,4 @@ class TestCpymadAperture(unittest.TestCase):
 
 
 if __name__ == '__main__':
-    suite = unittest.TestLoader().loadTestsFromTestCase(TestCpymadAperture)
-    unittest.TextTestRunner(verbosity=1).run(suite)
+    unittest.main()
