@@ -9,5 +9,4 @@ class TestTFS(unittest.TestCase):
 
 
 if __name__ == '__main__':
-    suite = unittest.TestLoader().loadTestsFromTestCase(TestTFS)
-    unittest.TextTestRunner(verbosity=2).run(suite)
+    unittest.main()
